@@ -1,27 +1,34 @@
 /*
  * Test suite for xmalloc and family.
  *
- * Copyright 2004, 2005, 2006
- *     by Internet Systems Consortium, Inc. ("ISC")
- * Copyright 1991, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
- *     2003 by The Internet Software Consortium and Rich Salz
+ * The canonical version of this file is maintained in the rra-c-util package,
+ * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
  *
- * This code is derived from software contributed to the Internet Software
- * Consortium by Rich Salz.
+ * Copyright 2000, 2001, 2006 Russ Allbery <rra@stanford.edu>
+ * Copyright 2008
+ *     The Board of Trustees of the Leland Stanford Junior University
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
+#line 1 "xmalloc.c"
+
 #include <config.h>
 #include <portable/system.h>
 
@@ -32,7 +39,8 @@
 /* Linux requires sys/time.h be included before sys/resource.h. */
 #include <sys/resource.h>
 
-#include <util/util.h>
+#include <util/messages.h>
+#include <util/xmalloc.h>
 
 
 /*
@@ -71,9 +79,9 @@ test_malloc(size_t size)
 
 
 /*
- * Allocate half the memory given, write to it, then reallocate to the desired
- * size, writing to the rest and then checking it all.  Returns true on
- * success, false on any failure.
+ * Allocate 10 bytes of memory given, write to it, then reallocate to the
+ * desired size, writing to the rest and then checking it all.  Returns true
+ * on success, false on any failure.
  */
 static int
 test_realloc(size_t size)
@@ -81,20 +89,19 @@ test_realloc(size_t size)
     char *buffer;
     size_t i;
 
-    buffer = xmalloc(size / 2);
+    buffer = xmalloc(10);
     if (buffer == NULL)
         return 0;
-    if (size / 2 > 0)
-        memset(buffer, 1, size / 2);
+    memset(buffer, 1, 10);
     buffer = xrealloc(buffer, size);
     if (buffer == NULL)
         return 0;
     if (size > 0)
-        memset(buffer + size / 2, 2, size - size / 2);
-    for (i = 0; i < size / 2; i++)
+        memset(buffer + 10, 2, size - 10);
+    for (i = 0; i < 10; i++)
         if (buffer[i] != 1)
             return 0;
-    for (i = size / 2; i < size; i++)
+    for (i = 10; i < size; i++)
         if (buffer[i] != 2)
             return 0;
     free(buffer);
@@ -195,6 +202,8 @@ test_asprintf(size_t size)
     string[size - 1] = '\0';
     status = xasprintf(&copy, "%s", string);
     free(string);
+    if (status < 0)
+        return 0;
     for (i = 0; i < size - 1; i++)
         if (copy[i] != 42)
             return 0;
@@ -235,6 +244,8 @@ test_vasprintf(size_t size)
     string[size - 1] = '\0';
     status = xvasprintf_wrapper(&copy, "%s", string);
     free(string);
+    if (status < 0)
+        return 0;
     for (i = 0; i < size - 1; i++)
         if (copy[i] != 42)
             return 0;
@@ -256,7 +267,6 @@ main(int argc, char *argv[])
     size_t limit = 0;
     int willfail = 0;
     unsigned char code;
-    struct rlimit rl;
 
     if (argc < 3)
         die("Usage error.  Type, size, and limit must be given.");
@@ -269,27 +279,6 @@ main(int argc, char *argv[])
     if (limit == 0 && errno != 0)
         sysdie("Invalid limit");
 
-    /*
-     * If a memory limit was given and we can set memory limits, set it.
-     * Otherwise, exit 2, signalling to the driver that the test should be
-     * skipped.  We do this here rather than in the driver due to some
-     * pathological problems with Linux (setting ulimit in the shell caused
-     * the shell to die).
-     */
-    if (limit > 0) {
-#if HAVE_SETRLIMIT && defined(RLIMIT_DATA)
-        rl.rlim_cur = limit;
-        rl.rlim_max = limit;
-        if (setrlimit(RLIMIT_DATA, &rl) < 0) {
-            syswarn("Can't set data limit to %lu", (unsigned long) limit);
-            exit(2);
-        }
-#else
-        warn("Data limits aren't supported.");
-        exit(2);
-#endif
-    }
-
     /* If the code is capitalized, install our customized error handler. */
     code = argv[1][0];
     if (isupper(code)) {
@@ -303,11 +292,47 @@ main(int argc, char *argv[])
      * that the test should be skipped.
      */
     max = size;
-    if (code == 's' || code == 'n' || code == 'a' || code == 'v')
-        max *= 2;
+    if (code == 's' || code == 'n' || code == 'a' || code == 'v') {
+        max += size;
+        if (limit > 0)
+            limit += size;
+    }
     if (limit > 0 && max > limit)
         willfail = 2;
 
+    /*
+     * If a memory limit was given and we can set memory limits, set it.
+     * Otherwise, exit 2, signalling to the driver that the test should be
+     * skipped.  We do this here rather than in the driver due to some
+     * pathological problems with Linux (setting ulimit in the shell caused
+     * the shell to die).
+     */
+    if (limit > 0) {
+#if HAVE_SETRLIMIT && defined(RLIMIT_AS)
+        struct rlimit rl;
+        void *tmp;
+
+        rl.rlim_cur = limit;
+        rl.rlim_max = limit;
+        if (setrlimit(RLIMIT_AS, &rl) < 0) {
+            syswarn("Can't set data limit to %lu", (unsigned long) limit);
+            exit(2);
+        }
+        if (size < limit || code == 'r') {
+            tmp = malloc(code == 'r' ? 10 : size);
+            if (tmp == NULL) {
+                syswarn("Can't allocate initial memory of %lu",
+                        (unsigned long) size);
+                exit(2);
+            }
+            free(tmp);
+        }
+#else
+        warn("Data limits aren't supported.");
+        exit(2);
+#endif
+    }
+
     switch (code) {
     case 'c': exit(test_calloc(size) ? willfail : 1);
     case 'm': exit(test_malloc(size) ? willfail : 1);
